/*********************************************************************
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * This file contains HCI inquery scan functionality
 */

#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <hci_scan.h>

/*
 * Function makes HCI inquery scan and fills in the BT addresses
 * and names
 * Returns pointer on success and NULL on error
 */
GSList* hci_scan(void)
{
    inquiry_info *info = NULL;
    char name[248];
    int dev_id = 0;
    int num_rsp = 10;
    int flags = 0;
    int length = 8;
    int dd, i;
    GSList* devList = NULL; //List of devices

    num_rsp = hci_inquiry(dev_id, length, num_rsp, NULL, &info, flags);
    if(num_rsp < 0)
	goto errout;

    if((dd = hci_open_dev(dev_id)) < 0)
    {
	free(info);
	goto errout;
    }

    for(i = 0; i < num_rsp; i++)
    {
	memset(name, 0, sizeof(name));
	if(hci_read_remote_name(dd, &(info+i)->bdaddr, sizeof(name), name, 100000) < 0)
	    strcpy(name, "n/a");
	Device* dev = (Device*)g_malloc(sizeof(Device));
	if (!dev)
	    break;
	memset(dev, 0, sizeof(Device));
	ba2str(&(info+i)->bdaddr, dev->addr);
	dev->name = g_strdup(name);
	devList = g_slist_append(devList, dev);
    }
    close(dd);
    free(info);
errout:
    return devList;
}

/*
 * Function gets the list of device MAC addresses and tries to get 
 * Their names
 * param alist -- the list of MAC addresses
 * Returns pointer on success and NULL on error
 */
GSList* hci_scan_names(GSList* alist)
{
    inquiry_info *info = NULL;
    char name[248];
    int dev_id = 0;
    int length = 8;
    int dd;
    GSList* devList = NULL; //List of devices
    GSList* aitem; //item in the address list
    bdaddr_t bdaddr;

    if((dd = hci_open_dev(dev_id)) < 0)
    {
	free(info);
	goto errout;
    }

    for(aitem = alist; aitem; aitem = aitem->next)
    {
	if (!aitem->data)
	    continue;
	memset(name, 0, sizeof(name));
	str2ba((char*)aitem->data, &bdaddr);
	if(hci_read_remote_name(dd, &bdaddr, sizeof(name), name, 100000) < 0)
	    strcpy(name, "n/a");
	Device* dev = (Device*)g_malloc(sizeof(Device));
	if (!dev)
	    break;
	memset(dev, 0, sizeof(Device));
	memcpy(dev->addr, aitem->data, 18);
	dev->name = g_strdup(name);
	devList = g_slist_append(devList, dev);
    }
    close(dd);
    free(info);
errout:
    return devList;
}

//eof
