/*
 * This is the main file of the OSSO ObexPut application
 */

#include <stdlib.h>
#include <bluetooth/bluetooth.h>
#include <sys/ioctl.h>
#include <bluetooth/hci.h>
#include <bluetooth/rfcomm.h>
#include <gw-obex.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include "hci_scan.h"
#include "sdp_search.h"

//Common defines
const char* devName = "/dev/rfcomm"; //Communication device name

/*
 * Get bluetooth socket
 * @return socket on success -1 on error
 */
static int get_sock(void) 
{
    int sock;
    int err = -1;
    do 
    {
	sock = socket(AF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM);
	err  = errno;
    } while(sock == -1 && err == ENFILE);
    return sock;
}

/*
 * Return the connected socket for the channel specified
 * Return socket or -1 on error
 */
int get_connected_sock(char dest[18], int channel) 
{
    int sock;
    int r;
    int err;
    do {
	sock = get_sock();
	struct sockaddr_rc addr;
	memset(&addr, 0, sizeof(addr));

	addr.rc_family = AF_BLUETOOTH;
	addr.rc_channel = channel;
	str2ba(dest, &addr.rc_bdaddr);

	r = connect(sock, (struct sockaddr *)&addr, sizeof(addr));
	err = errno;
	if(r != 0) {
	    int a = close(sock);
	    if (a < 0) 
		break;
	    usleep(10);
	}
    } while(r != 0 && err == EBUSY);

    if (r < 0)
	return -1;
    return sock;
}


/* 
 * Function sends a file to the destination
 * Returns 0 on success -1 on error
 */
int obexput(const char* filename, char* addr)
{
    GwObex* obexConn; //OBEX connection
    gint err; //error code
    int btfd; //bluetooth socket connection
    int channel; //bluetooth channel
    int res = 0; //transfer result
    const char* fileOnly; //Only a file name without a path

    fileOnly = strrchr(filename, '/');
    if (!fileOnly)
	fileOnly = filename;
    else
	fileOnly++;

    channel = get_obex_channel(addr);
    if (channel < 0)
    {
	res = -1;
	goto errout;
    }
    btfd = get_connected_sock(addr, channel);
    if (btfd < 0)
    {
	res = -1;
	goto errout;
    }
	
    obexConn = gw_obex_setup_fd(btfd, NULL, 0, NULL, &err);
    if (!obexConn)
    {
	res = -1;
	goto errout;
    }
    if (!gw_obex_put_file(obexConn, filename, fileOnly, "text/html", &err))
	res = -1;
    gw_obex_close(obexConn);
    close(btfd);
errout:
    return res;
}
//eof
